<?php
/**
 * Locale Auto Switch
 * 
 * @package    LocaleAutoSwitch
 * @subpackage LocaleAutoSwitch Management screen
    Copyright (c) 2018- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

$localeautoswitchadmin = new LocaleAutoSwitchAdmin();

class LocaleAutoSwitchAdmin {

	private $csvFileName;
	private $csvImportFileName;
	private $csvUrlName;
	private $upload_dir;
	private $languages;

	/* ==================================================
	 * Construct
	 * @since	1.00
	 */
	public function __construct() {

		$wp_uploads = wp_upload_dir();
		$relation_path_true = strpos($wp_uploads['baseurl'], '../');
		if ( $relation_path_true > 0 ) {
			$basepath = substr($wp_uploads['baseurl'], 0, $relation_path_true);
			$upload_url = $this->realurl($basepath, $relationalpath);
			$this->upload_dir = wp_normalize_path(realpath($wp_uploads['basedir']));
		} else {
			$upload_url = $wp_uploads['baseurl'];
			$this->upload_dir = wp_normalize_path($wp_uploads['basedir']);
		}
		if(is_ssl()){
			$upload_url = str_replace('http:', 'https:', $upload_url);
		}
		$this->upload_dir = untrailingslashit($this->upload_dir);
		$upload_url = untrailingslashit($upload_url);
		$this->csvFileName = $this->upload_dir.'/locale-wp-br-'.date("Y-m-d").'.csv';
		$this->csvUrlName = $upload_url.'/locale-wp-br-'.date("Y-m-d").'.csv';
		$this->languages = get_available_languages();
		$this->languages[] = 'en_US';

		add_action( 'admin_menu', array($this, 'plugin_menu'));
		add_action( 'admin_enqueue_scripts', array($this, 'load_custom_wp_admin_style') );
		add_filter( 'plugin_action_links', array($this, 'settings_link'), 10, 2 );

	}

	/* ==================================================
	 * Add a "Settings" link to the plugins page
	 * @since	1.00
	 */
	public function settings_link( $links, $file ) {
		static $this_plugin;
		if ( empty($this_plugin) ) {
			$this_plugin = 'locale-auto-switch/localeautoswitch.php';
		}
		if ( $file == $this_plugin ) {
			$links[] = '<a href="'.admin_url('options-general.php?page=LocaleAutoSwitch').'">'.__( 'Settings').'</a>';
		}
			return $links;
	}

	/* ==================================================
	 * Settings page
	 * @since	1.00
	 */
	public function plugin_menu() {
		add_options_page( 'Locale Auto Switch Options', 'Locale Auto Switch', 'manage_options', 'LocaleAutoSwitch', array($this, 'plugin_options') );
	}

	/* ==================================================
	 * Add Css and Script
	 * @since	1.00
	 */
	public function load_custom_wp_admin_style() {
		if ($this->is_my_plugin_screen()) {
			wp_enqueue_style( 'jquery-responsiveTabs', plugin_dir_url( __DIR__ ).'css/responsive-tabs.css' );
			wp_enqueue_style( 'jquery-responsiveTabs-style', plugin_dir_url( __DIR__ ).'css/style.css' );
			wp_enqueue_script('jquery');
			wp_enqueue_script( 'jquery-responsiveTabs', plugin_dir_url( __DIR__ ).'js/jquery.responsiveTabs.min.js' );
			wp_enqueue_script( 'localeautoswitch-admin-js', plugin_dir_url( __DIR__ ).'js/jquery.localeautoswitch.admin.js', array('jquery') );
		}
	}

	/* ==================================================
	 * For only admin style
	 * @since	1.00
	 */
	private function is_my_plugin_screen() {
		$screen = get_current_screen();
		if (is_object($screen) && $screen->id == 'settings_page_LocaleAutoSwitch') {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	/* ==================================================
	 * Settings page
	 * @since	1.00
	 */
	public function plugin_options() {

		if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		if ( file_exists($this->csvFileName) ) {
			unlink($this->csvFileName);
		}

		if( !empty($_POST['localeautoswitch_admin_tabs']) ) {
			$settings_tabs = intval($_POST['localeautoswitch_admin_tabs']);
			$post_nonce_field = 'las_tabs'.$settings_tabs;
			if ( isset($_POST[$post_nonce_field]) && $_POST[$post_nonce_field] ) {
				if ( check_admin_referer( 'las_settings'.$settings_tabs, $post_nonce_field ) ) {
					$this->options_updated($settings_tabs);
				}
			}
		}

		$scriptname = admin_url('options-general.php?page=LocaleAutoSwitch');

		$wp_options_name = 'locale_auto_switch';

		$import_html = NULL;
		if ( isset($_POST['Import']) && $_POST['Import'] ) {
			if ( isset($_POST['locale_auto_switch_file_load']) && $_POST['locale_auto_switch_file_load'] ) {
				if ( check_admin_referer('las_file_load', 'locale_auto_switch_file_load') ) {
					if ( !empty($_FILES['filename']['name']) ) {
						$filename = $_FILES['filename']['tmp_name'];
						$name = basename($filename);
						move_uploaded_file($filename, $this->upload_dir.'/'.$name.'.csv');
						$csv_file = $this->upload_dir.'/'.$name.'.csv';
						update_option( $wp_options_name, json_encode($this->import($csv_file)) );
						unlink($csv_file);
						echo '<div class="notice notice-success is-dismissible"><ul><li>'.__('Settings').' --> '.__('Imported from CSV file.', 'local-auto-switch').'</li></ul></div>';
					}
				}
			} else {
				$import_html = '<hr>'."\n";
				$import_html .= '<form method="post" action="'.$scriptname.'" enctype="multipart/form-data">'."\n";
				$import_html .= wp_nonce_field('las_file_load', 'locale_auto_switch_file_load')."\n";
				$import_html .= '<input name="filename" type="file" size="80" />'."\n";
				$import_html .= get_submit_button( __('Import'), 'large', 'Import', FALSE );
				$import_html .= '</form>'."\n";
			}
		}
		?>

		<div class="wrap">
		<h2>Locale Auto Switch</h2>

		<div id="localeautoswitch-admin-tabs">
		  <ul>
		    <li><a href="#localeautoswitch-admin-tabs-1"><?php _e('Settings'); ?></a></li>
			<li><a href="#localeautoswitch-admin-tabs-2"><?php _e('Donate to this plugin &#187;'); ?></a></li>
		  </ul>

		  <div id="localeautoswitch-admin-tabs-1">
			<div class="wrap">

				<form method="post" id="localeautoswitch_settings_form" action="<?php echo $scriptname; ?>">
				<?php wp_nonce_field('las_settings1', 'las_tabs1'); ?>

				<h2><?php _e('Locale', 'locale-auto-switch'); ?></h2>
				<div style="display: block;padding:5px 5px">
				<strong><?php _e('Languages that are not installed will be forced to switch to English.', 'locale-auto-switch'); ?></strong>
				</div>
				<div style="display: block;padding:5px 5px">
				<strong><?php echo sprintf(__('The installed language is marked with %1$s.', 'locale-auto-switch'), '<span class="dashicons dashicons-translation"></span>'); ?></strong>
				</div>
				<div style="display: block;padding:5px 5px">
				<strong><?php _e('Browser can specify multiple items separated by "|".', 'locale-auto-switch'); ?></strong>
				</div>

				<div class="submit">
					<?php submit_button( __('Save Changes'), 'large', 'Submit', FALSE ); ?>
					<?php submit_button( __('Default'), 'large', 'Default', FALSE ); ?>
					<?php submit_button( __('Export to CSV', 'locale-auto-switch'), 'large', 'Export', FALSE ); ?>
					<?php
					submit_button( __('Import from CSV', 'locale-auto-switch'), 'large', 'Import', FALSE );
					?>
				</div>
				</form>
				<?php echo $import_html;
				if ( file_exists($this->csvFileName) ) {
					?>
					<hr>
					<button type="button" class="button button-large" onclick="location.href='<?php echo $this->csvUrlName; ?>'"><?php _e('Download Export File'); ?></button>
					<?php
				}
				?>
				<hr>
				<table>

				<?php
				$localeautoswitch_options = json_decode(get_option($wp_options_name), TRUE);
				$count = 0;
				foreach ( (array)$localeautoswitch_options as $localeautoswitch_option ) {
					if ( $count % 2 == 0 ) {
						$trstyle = 'background: #fff;';
					} else {
						$trstyle = 'background: #eee;';
					}
					?>
					<tr style="<?php echo $trstyle; ?>">
					<?php
						if ( $count == 0 ) {
							?>
							<td><strong><?php _e('English'); ?></strong>
							<input type="hidden" name="localeautoswitch_locale_en[<?php echo $count; ?>]" value="<?php echo $localeautoswitch_option['locale_en']; ?>" form="localeautoswitch_settings_form">
							</td>
							<td><strong><?php _e('Native', 'locale-auto-switch'); ?></strong>
							<input type="hidden" name="localeautoswitch_locale_na[<?php echo $count; ?>]" value="<?php echo $localeautoswitch_option['locale_na']; ?>" form="localeautoswitch_settings_form">
							</td>
							<td><strong>WordPress</strong>
							<input type="hidden" name="localeautoswitch_locale_wp[<?php echo $count; ?>]" value="<?php echo $localeautoswitch_option['locale_wp']; ?>" form="localeautoswitch_settings_form">
							</td>
							<td align="center"><strong><?php _e('Browser', 'locale-auto-switch'); ?></strong>
							<input type="hidden" name="localeautoswitch_locale_br[<?php echo $count; ?>]" value="<?php if(!empty($localeautoswitch_option['locale_br'])) echo stripslashes(implode('|', $localeautoswitch_option['locale_br'])); ?>" form="localeautoswitch_settings_form">
							</td>
							<?php
						} else {
							?>
							<td><?php echo $localeautoswitch_option['locale_en']; ?>
							<input type="hidden" name="localeautoswitch_locale_en[<?php echo $count; ?>]" value="<?php echo $localeautoswitch_option['locale_en']; ?>" form="localeautoswitch_settings_form">
							</td>
							<td><?php echo $localeautoswitch_option['locale_na']; ?>
							<input type="hidden" name="localeautoswitch_locale_na[<?php echo $count; ?>]" value="<?php echo $localeautoswitch_option['locale_na']; ?>" form="localeautoswitch_settings_form">
							</td>
							<td>
							<?php
							if ( in_array($localeautoswitch_option['locale_wp'], $this->languages) ) {
								?>
								<span class="dashicons dashicons-translation"></span>
								<?php
							}
							echo $localeautoswitch_option['locale_wp']; ?>
							<input type="hidden" name="localeautoswitch_locale_wp[<?php echo $count; ?>]" value="<?php echo $localeautoswitch_option['locale_wp']; ?>" form="localeautoswitch_settings_form">
							</td>
							<td>
							<input type="text" name="localeautoswitch_locale_br[<?php echo $count; ?>]" value="<?php if(!empty($localeautoswitch_option['locale_br'])) echo stripslashes(implode('|', $localeautoswitch_option['locale_br'])); ?>" style="width: 160px;" form="localeautoswitch_settings_form">
							</td>
							<?php
						}
						?>
					</tr>

					<?php
					++$count;
				}
				?>
				<input type="hidden" name="localeautoswitch_admin_tabs" value="1" form="localeautoswitch_settings_form">
				</table>

			</div>
		  </div>

		  <div id="localeautoswitch-admin-tabs-2">
			<div class="wrap">
			<?php $this->credit(); ?>
			</div>
		  </div>

		</div>
		</div>
		<?php

	}

	/* ==================================================
	 * Credit
	 */
	private function credit() {

		$plugin_name = NULL;
		$plugin_ver_num = NULL;
		$plugin_path = plugin_dir_path( __DIR__ );
		$plugin_dir = untrailingslashit($plugin_path);
		$slugs = explode('/', $plugin_dir);
		$slug = end($slugs);
		$files = scandir($plugin_dir);
		foreach ($files as $file) {
			if($file == '.' || $file == '..' || is_dir($plugin_path.$file)){
				continue;
			} else {
				$exts = explode('.', $file);
				$ext = strtolower(end($exts));
				if ( $ext === 'php' ) {
					$plugin_datas = get_file_data( $plugin_path.$file, array('name'=>'Plugin Name', 'version' => 'Version') );
					if ( array_key_exists( "name", $plugin_datas ) && !empty($plugin_datas['name']) && array_key_exists( "version", $plugin_datas ) && !empty($plugin_datas['version']) ) {
						$plugin_name = $plugin_datas['name'];
						$plugin_ver_num = $plugin_datas['version'];
						break;
					}
				}
			}
		}
		$plugin_version = __('Version:').' '.$plugin_ver_num;
		$faq = __('https://wordpress.org/plugins/'.$slug.'/faq', $slug);
		$support = 'https://wordpress.org/support/plugin/'.$slug;
		$review = 'https://wordpress.org/support/view/plugin-reviews/'.$slug;
		$translate = 'https://translate.wordpress.org/projects/wp-plugins/'.$slug;
		$facebook = 'https://www.facebook.com/katsushikawamori/';
		$twitter = 'https://twitter.com/dodesyo312';
		$youtube = 'https://www.youtube.com/channel/UC5zTLeyROkvZm86OgNRcb_w';
		$donate = __('https://riverforest-wp.info/donate/', $slug);

		?>
		<span style="font-weight: bold;">
		<div>
		<?php echo $plugin_version; ?> | 
		<a style="text-decoration: none;" href="<?php echo $faq; ?>" target="_blank"><?php _e('FAQ'); ?></a> | <a style="text-decoration: none;" href="<?php echo $support; ?>" target="_blank"><?php _e('Support Forums'); ?></a> | <a style="text-decoration: none;" href="<?php echo $review; ?>" target="_blank"><?php _e('Reviews', 'media-from-ftp'); ?></a>
		</div>
		<div>
		<a style="text-decoration: none;" href="<?php echo $translate; ?>" target="_blank"><?php echo sprintf(__('Translations for %s'), $plugin_name); ?></a> | <a style="text-decoration: none;" href="<?php echo $facebook; ?>" target="_blank"><span class="dashicons dashicons-facebook"></span></a> | <a style="text-decoration: none;" href="<?php echo $twitter; ?>" target="_blank"><span class="dashicons dashicons-twitter"></span></a> | <a style="text-decoration: none;" href="<?php echo $youtube; ?>" target="_blank"><span class="dashicons dashicons-video-alt3"></span></a>
		</div>
		</span>

		<div style="width: 250px; height: 180px; margin: 5px; padding: 5px; border: #CCC 2px solid;">
		<h3><?php _e('Please make a donation if you like my work or would like to further the development of this plugin.', $slug); ?></h3>
		<div style="text-align: right; margin: 5px; padding: 5px;"><span style="padding: 3px; color: #ffffff; background-color: #008000">Plugin Author</span> <span style="font-weight: bold;">Katsushi Kawamori</span></div>
		<button type="button" style="margin: 5px; padding: 5px;" onclick="window.open('<?php echo $donate; ?>')"><?php _e('Donate to this plugin &#187;'); ?></button>
		</div>

		<?php

	}

	/* ==================================================
	 * Update wp_options table.
	 * @param	string	$tabs
	 * @since	1.00
	 */
	private function options_updated($tabs){

		$wp_options_name = 'locale_auto_switch';

		switch ($tabs) {
			case 1:
				if ( !empty($_POST['Default']) ) {
					$csv_file = plugin_dir_path( __DIR__ ).'data/locale-wp-br.csv';
					update_option( $wp_options_name, json_encode($this->import($csv_file)) );
					echo '<div class="notice notice-success is-dismissible"><ul><li>'.__('Settings').' --> '.__('Default').' --> '.__('Changes saved.').'</li></ul></div>';
				} else if ( !empty($_POST['Export']) ) {
					$locale_wp_br_s = json_decode(get_option($wp_options_name), TRUE);
					$csv = NULL;
					foreach ( (array)$locale_wp_br_s as $locale_wp_br ) {
						$csv .= $locale_wp_br['locale_en'].','.$locale_wp_br['locale_na'].','.$locale_wp_br['locale_wp'].',';
						if ( !empty($locale_wp_br['locale_br']) ) {
							$csv .= stripslashes(implode('|', $locale_wp_br['locale_br']));
						}
						$csv .= "\n";
					}
					file_put_contents($this->csvFileName, pack('C*',0xEF,0xBB,0xBF)); //UTF-8 BOM
					file_put_contents($this->csvFileName, $csv, FILE_APPEND | LOCK_EX);
					echo '<div class="notice notice-success is-dismissible"><ul><li>'.__('Settings').' --> '.__('Exported a CSV file. Please download it.', 'locale-auto-switch').'</li></ul></div>';
				} else if ( !empty($_POST['Import']) ) {
					// Skip
				} else {
					if ( !empty($_POST['localeautoswitch_locale_br']) ) {
						$locale_en = $this->sanitize_array($_POST['localeautoswitch_locale_en']);
						$locale_na = $this->sanitize_array($_POST['localeautoswitch_locale_na']);
						$locale_wp = $this->sanitize_array($_POST['localeautoswitch_locale_wp']);
						$locale_brs = $this->sanitize_array($_POST['localeautoswitch_locale_br']);
						$count = 0;
						for ( $i = 0; $i <= count($locale_wp)-1; $i++ ){
							if ( !empty($locale_brs[$i]) ) {
								$locale_br = explode('|', sanitize_text_field($locale_brs[$i]));
							} else {
								$locale_br = NULL;
							}
							$locale_auto_switch_tbl[$i] = array(
											'locale_en' => stripslashes($locale_en[$i]),
											'locale_na' => stripslashes($locale_na[$i]),
											'locale_wp' => $locale_wp[$i],
											'locale_br' => $locale_br
										);
						}
						update_option( $wp_options_name, json_encode($locale_auto_switch_tbl) );
						echo '<div class="notice notice-success is-dismissible"><ul><li>'.__('Settings').' --> '.__('Changes saved.').'</li></ul></div>';
					}
				}
				break;
		}

		return;

	}

	/* ==================================================
	* Sanitize Array
	* @param	string	$csv_file
	* @return	array	$locale_auto_switch_tbl
	* @since	1.00
	*/
	private function import($csv_file) {

		$locale_auto_switch_tbl = array();
		$f = fopen($csv_file, 'r');
		while( $line = fgetcsv($f) ){
			$locale_br = explode('|', $line[3]);
			$locale_auto_switch_tbl[] = array(
								'locale_en' => $line[0],
								'locale_na' => $line[1],
								'locale_wp' => $line[2],
								'locale_br' => $locale_br
							);
		}
		fclose($f);

		return $locale_auto_switch_tbl;

	}

	/* ==================================================
	* Sanitize Array
	* @param	array	$a
	* @return	string	$_a
	* @since	1.00
	*/
	private function sanitize_array($a) {

		$_a = array();
		foreach($a as $key=>$value) {
			if ( is_array($value) ) {
				$_a[$key] = $this->sanitize_array($value);
			} else {
				$_a[$key] = htmlspecialchars($value);
			}
		}

		return $_a;

	}

}

?>